package com.flyfish.framework.beans.meta;

import com.flyfish.framework.beans.annotations.RestBean;
import lombok.Data;
import org.springframework.beans.BeanUtils;

import java.beans.PropertyDescriptor;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * bean的属性描述
 * 对应js的类型
 *
 * @author wangyu
 */
@Data
public class BeanProperty {

    // 属性名称
    private String name;

    // bean属性的类型，js类型
    private BeanPropertyType type;

    // 属性
    private Map<String, Object> props;

    // 类型为object时，拥有子表单
    private List<BeanProperty> children;

    /**
     * 类加载并解析属性
     *
     * @return 结果
     */
    public static List<BeanProperty> from(PropertyDescriptor[] descriptors) {
        return Arrays.stream(descriptors)
                .filter(descriptor -> !"class".equals(descriptor.getName()))
                .map(BeanProperty::form)
                .collect(Collectors.toList());
    }

    /**
     * 来自属性解释器构造
     * 支持对象嵌套
     *
     * @param descriptor 解释器
     * @return 结果
     */
    public static BeanProperty form(PropertyDescriptor descriptor) {
        Class<?> clazz = descriptor.getPropertyType();
        BeanProperty property = new BeanProperty();
        property.setName(descriptor.getName());
        property.setType(BeanPropertyType.of(descriptor));
        if (property.getType() == BeanPropertyType.OBJECT) {
            // 有子bean注解才处理
            if (clazz.isAnnotationPresent(SubBean.class)) {
                List<BeanProperty> children = from(BeanUtils.getPropertyDescriptors(clazz));
                property.setChildren(children);
            }
        } else if (property.getType() == BeanPropertyType.DB_REF) {
            // 当存在db-ref时，解析为动态数据源
            RestBean restBean = clazz.getAnnotation(RestBean.class);
            String uri = restBean.value();
            Map<String, Object> props = new HashMap<>();
            props.put("uri", uri);
            property.setProps(props);
        }
        return property;
    }
}
